import { Format } from './Format';
import { registerMethodKeys } from './constants';
export class GenerateAppEntryFile {
  static addSpaceToFileContent(text) {
    this.fileContent += text;
    this.fileContent += `\n\n`;
  }

  static async handleAddFileContentByApplyHook({
    key,
    api,
    formatFn = v => v
  }) {
    const values = await api.applyPlugins({
      key
    });

    if (values.length) {
      this.addSpaceToFileContent(values.map(v => formatFn(v)).join('\n'));
    }
  }

  static async getContainerRenderFunctionNames() {
    return [...new Set(this.fileContent.match(/(?<=function\s)\S+(?=\s?\()/g) || [])];
  }

  static result() {
    const content = this.fileContent;
    this.fileContent = '';
    return content;
  }

  static async run(api) {
    await this.handleAddFileContentByApplyHook({
      key: registerMethodKeys.addImportToAppEntryFile,
      api,
      formatFn: Format.addImportToAppEntryFile
    });
    await this.handleAddFileContentByApplyHook({
      key: registerMethodKeys.addContainerRenderToAppEntryFile,
      api
    });
    const renderFCs = await this.getContainerRenderFunctionNames();
    this.addSpaceToFileContent(`
const renderContainers = [${renderFCs.join(', ')}] as any[];

renderContainers.reduce((prev, current) => {
  if (!isFn(current)) {
    return prev;
  }

  return current(isFn(prev) ? prev() : prev);
});
    `);
    return this.result();
  }

}
GenerateAppEntryFile.fileContent = '';