"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = _default;

var _cloneDeep = _interopRequireDefault(require("lodash/cloneDeep"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const SEPARATOR = '^^^';

const routeToImportName = componentPath => componentPath.replace(/\//g, '').replace(/@/g, '').replace(/\./g, '').toLocaleUpperCase();

const pMinDelayImportName = 'pMinDelay';
const dynamicName = 'dynamic';
const dynamicWrapperVarName = 'dynamicWrapper';

function isFunctionComponent(component) {
  return /^\((.+)?\)(\s+)?=>/.test(component) || /^function([^\(]+)?\(([^\)]+)?\)([^{]+)?{/.test(component);
}

function _default(config) {
  const clonedRoutes = (0, _cloneDeep.default)(config.routes);
  let routeImportNameTemplate = '';
  let dynamicWrapperTemplate = '';

  function patchRoutes(routes, parentRoute) {
    routes.forEach(route => patchRoute(route, parentRoute));
  }

  function patchRoute(route, parentRoute) {
    if (route.component && !isFunctionComponent(route.component)) {
      const routeImportName = routeToImportName(route.component);

      if (!config.dynamicImport) {
        routeImportNameTemplate += `\nimport ${routeImportName} from '${route.component}';`;
      }

      route.component = [route.component, routeImportName].join(SEPARATOR);

      if (route.path) {
        if (parentRoute === null || parentRoute === void 0 ? void 0 : parentRoute.path) {
          if (parentRoute.path === route.path && !route.index) {
            route.index = true;
            delete route.path;
          }
        }
      }
    }

    if (route.routes) {
      patchRoutes(route.routes, route);
    }
  }

  function injectionDynamicImportWrapperFN() {
    if (!dynamicWrapperTemplate) {
      var _config$dynamicImport, _config$dynamicImport2;

      const delayText = ((_config$dynamicImport = config.dynamicImport) === null || _config$dynamicImport === void 0 ? void 0 : _config$dynamicImport.delay) ? `${pMinDelayImportName}(loadComp, ${config.dynamicImport.delay})` : 'loadComp';
      const LoadingText = !((_config$dynamicImport2 = config.dynamicImport) === null || _config$dynamicImport2 === void 0 ? void 0 : _config$dynamicImport2.loading) ? '' : `, { fallback: <LoadingComponent /> }`;
      dynamicWrapperTemplate = `const ${dynamicWrapperVarName} = (loadComp: PromiseLike<any>) => {
  const Component = ${dynamicName}(() => ${delayText}${LoadingText});

  return <Component />;
}`;
    }
  }

  function init() {
    var _config$dynamicImport3;

    if ((_config$dynamicImport3 = config.dynamicImport) === null || _config$dynamicImport3 === void 0 ? void 0 : _config$dynamicImport3.delay) {
      routeImportNameTemplate += `\nimport ${pMinDelayImportName} from 'p-min-delay';`;
    }

    injectionDynamicImportWrapperFN();
    patchRoutes(clonedRoutes);
  }

  function replacer(key, value) {
    switch (key) {
      case 'component':
        if (isFunctionComponent(value)) return value;
        const [component, routeImportName] = value.split(SEPARATOR);
        return config.dynamicImport ? `${dynamicWrapperVarName}(import('${component}'))` : routeImportName;

      default:
        return value;
    }
  }

  init();
  const routes = JSON.stringify(clonedRoutes, replacer, 2).replace(/\"component\": (\"(.+?)\")/g, (global, m1, m2) => {
    return `"element": ${m2.replace(/\^/g, '"')}`;
  }).replace(/\"routes\":/g, '"children":').replace(/\\r\\n/g, '\r\n').replace(/\\n/g, '\r\n');
  return {
    importNameTemplate: routeImportNameTemplate.replace('\n', '') + `\n\n${dynamicWrapperTemplate}`,
    routes
  };
}