"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.GenerateFile = void 0;

var _path = require("path");

var _fsExtra = require("fs-extra");

var Mustache = _interopRequireWildcard(require("mustache"));

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

class GenerateFile {
  constructor(server) {
    this.generateFileQueue = [];
    this.server = server;
  }

  addGenerateFileToQueue(opt) {
    this.generateFileQueue.push(opt);
    return this;
  }

  copyFile(src, dest) {
    return (0, _fsExtra.copy)(src, dest);
  }

  readTplFile(...filePath) {
    return (0, _fsExtra.readFileSync)((0, _path.join)(...filePath), 'utf-8');
  }

  renderTpl(content, view) {
    return Mustache.render(content, view);
  }

  outputFile(f, c) {
    return (0, _fsExtra.outputFile)(f, c, 'utf-8');
  }

  async runGenerateFileQueue() {
    if (!this.generateFileQueue.length) {
      return undefined;
    }

    return Promise.all(this.generateFileQueue.map(item => this.outputFile(item.path, item.content))).then(() => {
      this.generateFileQueue = [];
    });
  }

  async removeTmpFile(path) {
    const filePath = this.server.resolveTmpPath(path);
    return (0, _fsExtra.existsSync)(filePath) ? (0, _fsExtra.remove)(filePath) : true;
  }

  async writeFile({
    path,
    content,
    isPrettier
  }) {
    const filePath = this.server.resolveTmpPath(path);

    if ((0, _fsExtra.existsSync)(filePath) && (0, _fsExtra.readFileSync)(filePath, 'utf-8') === content) {
      return false;
    }

    await this.outputFile(filePath, content);

    if (isPrettier) {
      this.server.prettier.addPrettierFile(filePath);
    }

    return true;
  }

}

exports.GenerateFile = GenerateFile;